from pathlib import Path
from typing import BinaryIO, Union


def is_silk_data(raw: bytes) -> bool:
    if len(raw) > 10:
        if raw[:10] == b"\x02#!SILK_V3":
            return True
    return False


def get_file(file: Union[str, BinaryIO]) -> BinaryIO:
    if isinstance(file, str):
        path = Path(file)
        if not path.is_file():
            raise FileNotFoundError(file)
        return open(file, "rb")
    elif isinstance(file, BinaryIO):
        return file
    else:
        raise TypeError(file)



import asyncio

from os import cpu_count
from io import BytesIO
from typing import Union, BinaryIO
from concurrent.futures import ThreadPoolExecutor

from _pysilk import silkEncode, silkDecode
from .utils import get_file, is_silk_data
from .wav import wav2pcm, pcm2wav


_LOOP = asyncio.get_event_loop()
_EXECUTOR = ThreadPoolExecutor(cpu_count())


def encode(data: bytes, sample_rate=24000) -> bytes:
    if data[8:12] == b"WAVE":
        return silkEncode(
            wav2pcm(BytesIO(data)), sample_rate
        )
    return silkEncode(data, sample_rate)


def encode_file(target: Union[str, BinaryIO], sample_rate=24000) -> bytes:
    fd = get_file(target)
    try:
        return encode(fd.read(), sample_rate)
    finally:
        fd.close()


def decode(silk_data: bytes, sample_rate=24000, to_wav=False) -> bytes:
    if is_silk_data(silk_data):
        if to_wav:
            return silkDecode(silk_data, sample_rate)
        else:
            return pcm2wav(
                BytesIO(silkDecode(silk_data, sample_rate))
            )
    else:
        raise ValueError("Not a valid silk_data")


def decode_file(silk_file: Union[str, BinaryIO], sample_rate=24000, to_wav=False) -> bytes:
    fd = get_file(silk_file)
    try:
        return decode(fd.read(), sample_rate, to_wav)
    finally:
        fd.close()


async def async_encode(data: bytes, sample_rate=24000) -> bytes:
    return await _LOOP.run_in_executor(_EXECUTOR, encode, data, sample_rate)


async def async_decode(silk_data: bytes, sample_rate=24000, to_wav=False) -> bytes:
    return await _LOOP.run_in_executor(_EXECUTOR, decode, silk_data, sample_rate, to_wav)


async def async_encode_file(target: Union[str, BinaryIO], sample_rate=24000) -> bytes:
    return await _LOOP.run_in_executor(_EXECUTOR, encode_file, target, sample_rate)


async def async_decode_file(silk_file: Union[str, BinaryIO], sample_rate=24000, to_wav=False) -> bytes:
    return await _LOOP.run_in_executor(_EXECUTOR, decode_file, silk_file, sample_rate, to_wav)



import wave
from io import BytesIO
from typing import BinaryIO


def pcm2wav(fobj: BinaryIO, frame_rate=24000, *, ac=1) -> bytes:
    res = BytesIO()
    with wave.Wave_write(res) as wav:
        wav.setframerate(frame_rate)
        wav.setsampwidth(ac * 2)
        wav.setnchannels(ac)
        for x in fobj:
            wav.writeframes(x)
    return res.getvalue()


def wav2pcm(fobj: BinaryIO) -> bytes:
    res = BytesIO()
    with wave.Wave_read(fobj) as wav:
        if wav.getcomptype() != "NONE":
            raise wave.Error("unsupport with-compress wave file")
        while True:
            # TODO: to_single_channel
            bl = wav.readframes(512)
            if bl:
                res.write(bl)
            else:
                break
    return res.getvalue()



import glob

from setuptools import setup
# Available at setup time due to pyproject.toml
try:
    from pybind11.setup_helpers import Pybind11Extension as Extension
except ImportError:
    from setuptools import Extension

__version__ = "1.2.2"

setup(
    name="pysilk-mod",
    version=__version__,
    author="DCZYewen",
    author_email="contact@basicws.net",
    url="https://github.com/DCZYewen/Python-Silk-Module",
    description="Python silk decode/encoder",
    long_description="Python silk decode/encoder bindings using pybind11",
    packages=["src/pysilk"],
    requires=["pybind11"],
    zip_safe=False,
    ext_modules=[
        Extension(
            "_pysilk", ["src/silk/_pysilk.cpp", "src/silk/codec.cpp", *glob.glob("src/silk/src/*.c")],
            include_dirs=["src/silk/interface"],
            define_macros=[('VERSION_INFO', __version__)]
        )
    ]
)
